function GMMstrct = trainGMM(Z, pars)
% function GMMstrct = trainGMM(Z, pars)
%
% Maximization step
% Version 1.0
% copyright by Hadas Benisty (2012)
% Technion
% contact hadasbe@tx.technion.ac.il
%

Z = Z.';
if pars.diagonalCov
    GMMsourceTmpFile = fullfile(pars.GMMdir, 'GMMsourceDiag.mat');
else
    GMMsourceTmpFile = fullfile(pars.GMMdir, 'GMMsource.mat');
end
if ~exist(GMMsourceTmpFile, 'file') || pars.overwrite
    disp('Evaluate a pertubation to be added to the diagonal');
    ord = size(Z, 1);
    Gconst=power(2*pi,-ord);
    pert = evalDiagPert(Z);
    disp('Evaluate the initial clusters using kmeans');
    GMMstrct = initEMkmeans(Z, pars.m, pert, pars.diagonalCov);
    
    disp('Intialize p(m|x_t)');
    P = expectationStep(GMMstrct, Z);
    disp('Begin iterating EM');
    L = zeros(pars.maxiterations, 1);
    Lprev=1;
    % main EM loop
    for iter = 1:pars.maxiterations
        GMMstrct = maximizationStep(GMMstrct, P, Z, pert, pars.diagonalCov);
        [L(iter), P] = evalLikelihood(Z, GMMstrct, Gconst);
        % check if we can stop
        disp([num2str(abs((L(iter)-Lprev)/Lprev)) ' ' num2str(pars.LLth)]);
        if abs((L(iter)-Lprev)/Lprev)<pars.LLth
            disp('EM converged');
            break;
        else
            Lprev=L(iter);
        end;
    end
    GMMstrct(1).L = L;
    save(GMMsourceTmpFile,'GMMstrct');
    
else
    load(GMMsourceTmpFile);
end